import sys
from django.test import TestCase

error_while_importing = None

try:
    from easymode import *
    from easymode.admin import *
    from easymode.admin.forms import *
    from easymode.admin.models import *
    from easymode.debug import *
    from easymode.easypublisher import *
    from easymode.easypublisher.admin import *
    from easymode.easypublisher.models import *
    from easymode.easypublisher.response import *
    from easymode.easypublisher.templatetags import *
    from easymode.easypublisher.urls import *
    from easymode.easypublisher.utils import *
    from easymode.easypublisher.views import *
    from easymode.i18n import *
    from easymode.i18n.admin import *
    from easymode.management import *
    from easymode.middleware import *
    from easymode.management.commands import *
    from easymode.templatetags import *
    from easymode.tree import *
    from easymode.tree.admin import *
    from easymode.tree.admin.widgets import *
    from easymode.urls import *
    from easymode.utils import *
    from easymode.views import *
    from easymode.xslt import *
except Exception as e:
    error_while_importing = e

__all__ = ('TestImportAll',)

class TestImportAll(TestCase):
    """Check if the import all works for every package in easymode"""

    def test_import_all(self):
        """All easymode packages should be importable with * without any errors""" 
        if error_while_importing is not None:
            self.fail("%s: %s" % (type(e).__name__, error_while_importing))


from django.db import models
from django.utils.translation import ugettext_lazy as _

from easymode.i18n.decorators import I18n
from easymode.tree.decorators import toxml

# Create your models here.

KINDS = (
    ('cow' , _("An animal that produces milk")),
    ('horse' , _("A unit used to measure car engine power")),
)

@toxml
@I18n('city', 'date')
class Foo(models.Model):
    bar = models.CharField(max_length=255, unique=True)
    barstool = models.TextField(max_length=4)
    website = models.URLField()
    address = models.CharField(max_length=32)
    city = models.CharField(max_length=40)
    date = models.DateField()
    
    def __unicode__(self):
        return u"%d. %s" % (self.id, self.bar)
        
    class Meta:
        ordering = ('id',)
    
    def natural_key(self):
        return (self.bar,)
        
class Bar(models.Model):
    foo = models.ForeignKey(Foo, related_name='bars')
    
    label = models.CharField(max_length=255)
    date = models.DateField()

    def __unicode__(self):
        return u"%d. %s" % (self.id, self.label)

    class Meta:
        ordering = ('id',)

class Baz(models.Model):
    """I am an independent global model"""
    baz = models.CharField(max_length=255)
    independancy_level = models.IntegerField()
    kind = models.CharField(max_length=255, choices=KINDS)
    
    def __unicode__(self):
        return u"%d. %s" % (self.id, self.baz)
    
    class Meta:
        ordering = ('id',)
    


"""
Contains a modeladmin class that shows links to all related items in its change_view.

If you want to use :class:`InvisibleModelAdmin` make sure easymode comes before
``django.contrib.admin`` in the ``INSTALLED_APPS`` because it has to 
override admin/index.html to make it work.
"""
from django.http import HttpResponseRedirect
from django.forms.models import  modelformset_factory
from django.core import urlresolvers
from django.conf import settings
from django.utils.encoding import force_unicode

from easymode.tree.introspection import get_foreign_key_desciptors
from easymode.tree.admin.formsets import VisiblePrimaryKeyFormset
from easymode.utils.languagecode import strip_language_code

if 'reversion' in settings.INSTALLED_APPS:
    from reversion.admin import VersionAdmin
    AdminBase = VersionAdmin
else:
    from django.contrib import admin
    AdminBase = admin.ModelAdmin

class _CanFindParentLink(object):
    """Adds function to find a link to a parent model"""
        
    def _get_parent_link(self, object_id):
        parent_link_data = {}
        if hasattr(self, 'parent_link'):
            parent_link = getattr(self.model, self.parent_link)
            instance = self.model.objects.get(pk=object_id)

            parent = parent_link.__get__(instance)
            parent_type_name = parent._meta.object_name.lower()
            parent_name = parent
            parent_id = str(parent_link.field.value_from_object(instance))

            info = (self.admin_site.name, self.model._meta.app_label, parent_type_name)
            
            parent_link_data['parent_model'] = strip_language_code(urlresolvers.reverse("%s:%s_%s_change" % info, args=[parent_id]))
            parent_link_data['parent_name'] = "%s %s" % (force_unicode(parent._meta.verbose_name), parent_name)
        
        return parent_link_data


class ForeignKeyAwareModelAdmin(AdminBase, _CanFindParentLink):
    """
    An admin class that display links to related items.
    
    This can be used for hierarchies of object deeper then 3
    levels, where :class:`InlineModelAdmin` can not be used anymore, but
    the parent/child relation is still there.
    
    usage::
    
        from easymode.tree.admin.relation import ForeignKeyAwareModelAdmin
        
        class SomeAdmin(ForeignKeyAwareModelAdmin):
            children = [SomeModelThatPointsToUs, AnotherModelThatPointsTous]
            invisible_in_admin = False
        
        admin.site.register(SomeModelWithLotsOfRelations, SomeAdmin)
    
    This will add the ``SomeModelThatPointsToUs`` and ``AnotherModelThatPointsTous`` to
    the ``SomeModelWithLotsOfRelations`` admin interface and you can add these children
    or edit them there.
    
    See :class:`InvisibleModelAdmin` if you want to hide the admin interface for ``SomeModelThatPointsToUs``
    and ``AnotherModelThatPointsTous`` admin interface from the admin listing.
    
    .. attribute:: auto_aware
    
        If ``auto_aware`` is true, the admin will find out for itself which
        models are child nodes, by inspecting it's managers.
        
    .. attribute:: parent_link
        
        To have sane breadcrumbs if the :class:`~easymode.tree.admin.relation.ForeignKeyAwareModelAdmin`
        is used as child of another :class:`~easymode.tree.admin.relation.ForeignKeyAwareModelAdmin`
        and make the save button return to the parent instead of the app listing, the
        ``parent_link`` should be set.
        
        It must be set to the *name* of the ``ForeignKey`` that points to the parent.
    
    .. attribute:: children
        
        If the order of the children should be changed or not all children should be
        displayed, you can specify the children manually.
        
        children should be set to a list of models that are child nodes of the model
        class that this admin class makes editable:
            
    .. attribute:: invisible_in_admin
    
        The :class:`~easymode.tree.admin.relation.ForeignKeyAwareModelAdmin` will not
        be shown in the admin listing if this value is ``True``. The default is ``True``.
    
    """
    change_form_template = 'tree/admin/change_form_with_related_links.html'
    
    invisible_in_admin = True
    auto_aware = True
    children = []
    
    _descriptor_cache = {}

    class Media:
        js = (
            'easymode/js/adminoverride.js',
        )
    
    def __init__(self, model, admin_site):
        super(ForeignKeyAwareModelAdmin, self).__init__(model, admin_site)

        # look up all foreign key descriptors
        descriptors = get_foreign_key_desciptors(self.model)
        self._descriptor_cache = dict([(x[1].related.model, x[0]) for x in descriptors])
        
        # if no children are set and autoaware is true, 
        # set the children
        if not self.children and self.auto_aware:
            self.children = [x[1].related.model for x in descriptors]
        
    def get_model_perms(self, request):
        perms = super(ForeignKeyAwareModelAdmin, self).get_model_perms(request)
        perms['invisible_in_admin'] = self.invisible_in_admin
        return perms
        
    def change_view(self, request, object_id, extra_context=None):
        inline_links = {}
        inline_links['extra_forms'] = self.extra_forms(object_id)

        # retrieve link to parent for breadcrumb path
        inline_links.update(self._get_parent_link(object_id))
        if extra_context:
            inline_links.update(extra_context)
            
        return super(ForeignKeyAwareModelAdmin, self).change_view(request, object_id, inline_links)

    def extra_forms(self, object_id):

        instance = self.model.objects.get(pk=object_id)
                
        extra_formsets = []

        for child in self.children:
            factory = modelformset_factory(child, extra=0, fields=['id'], formset=VisiblePrimaryKeyFormset)
            descriptor_name = self._descriptor_cache[child]
            descriptor = getattr(instance, descriptor_name)
            
            # create formset
            form = factory(queryset=descriptor.all())

            # this will find the name of the property in the model
            # the descriptor's inverse references
            try:
                field_name = descriptor.core_filters.keys().pop().split('__')[0]
            except Exception:
                field_name = instance._meta.object_name.lower()
            
            # find url for the + button
            url_descriptor = (self.admin_site.name, child._meta.app_label, child._meta.object_name.lower())
            url_pattern = '%s:%s_%s_add' % url_descriptor
            url = urlresolvers.reverse(url_pattern)
                
            #add properties to the formset            
            form.parent = instance            
            form.name = child.__name__.lower()
            form.title = child._meta.verbose_name_plural            
            form.addurl = "%s?%s=%s" % (strip_language_code(url), field_name, object_id)
            
            extra_formsets.append( form )            

        return extra_formsets
        


class InvisibleModelAdmin(AdminBase, _CanFindParentLink):
    """
    An admin class that can be used as admin for children
    of :class:`~easymode.tree.admin.relation.ForeignKeyAwareModelAdmin`. 
    
    This way they will be hidden in 
    the admin interface so they can only be accessed via ``ForeignKeyAwareModelAdmin``.
    usage::
        
        from django.db import models
        from django.contrib import admin
        from easymode.tree.admin.relation import *
        
        class Bar(models.Model):
            foo = models.ForeignKey(Foo, related_name='bars')
            label = models.CharField(max_length=255)
            
        class BarAdmin(InvisibleModelAdmin):
            model = Bar
            parent_link = 'foo'
    
        admin.site.register(Bar, BarAdmin)
        
    .. attribute:: parent_link
        
        When :class:`~easymode.tree.admin.relation.InvisibleModelAdmin` is used, it is nolonger
        displayed in the admin listing as an editable model. To have sane breadcrumbs
        and make the save button return to the parent instead of the app listing, the
        ``parent_link`` should be set.
        
        It must be set to the *name* of the ``ForeignKey`` that points to the parent.
    """
    change_form_template = 'tree/admin/change_form_with_parent_link.html'
    invisible_in_admin = True

    def get_model_perms(self, request):
        perms = super(InvisibleModelAdmin, self).get_model_perms(request)
        perms['invisible_in_admin'] = self.invisible_in_admin
        return perms
        
    def change_view(self, request, object_id, extra_context=None):

        # retrieve link to parent for breadcrumb path
        defaults = self._get_parent_link(object_id)

        if extra_context:
            defaults.update(extra_context)
        
        response = super(InvisibleModelAdmin, self).change_view(request, object_id, defaults)
        if response.get('Location', False) == '../':
            return HttpResponseRedirect(defaults.get('parent_model', '../'))
            
        return response



from django.contrib import admin

from easymode.i18n.admin.decorators import L10n
from easymode.tree.admin.relation import *
from easymode.easypublisher.admin import EasyPublisher

from foobar.models import *

@L10n
class FooAdmin(ForeignKeyAwareModelAdmin):
    """Admin class for the Foo model"""
    model = Foo
    invisible_in_admin = False
    
    fieldsets = (
        (None, {
            'fields': ('bar', 'barstool')
        }),
        ('An thingy', {
            'fields': ('website', 'city', 'address', 'date')
        }),
    )

class BarAdmin(InvisibleModelAdmin):
    model = Bar
    parent_link = 'foo'

class BazAdmin(EasyPublisher):
    model = Baz
    
admin.site.register(Foo, FooAdmin)
admin.site.register(Bar, BarAdmin)
admin.site.register(Baz, BazAdmin)


# Django settings for example project.
import os
import sys

ugettext = lambda s: s

DEBUG = True
TEMPLATE_DEBUG = DEBUG

PROJECT_DIR = os.path.dirname(__file__)

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASE_ENGINE = 'sqlite3'           # 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
DATABASE_NAME = 'example.sqlite'             # Or path to database file if using sqlite3.
DATABASE_USER = ''             # Not used with sqlite3.
DATABASE_PASSWORD = ''         # Not used with sqlite3.
DATABASE_HOST = ''             # Set to empty string for localhost. Not used with sqlite3.
DATABASE_PORT = ''             # Set to empty string for default. Not used with sqlite3.
# TEST_DATABASE_NAME = 'example_test_data.sqlite'
# DATABASE_NAME = TEST_DATABASE_NAME

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Chicago'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en'

LANGUAGES = (
    ('en', ugettext('English')),
    ('de', ugettext('German')),
    ('en-us', ugettext('usa')),
)


# gettext catalogs created from database data are
# stored here
LOCALE_DIR = '/tmp/'

FALLBACK_LANGUAGES = {
    'en':[],
    'de':['en'],
}

MSGID_LANGUAGE = 'en'

SITE_ID = 1

# a gettext catalog will only kept for a site, if the
# MASTER_SITE property is set to True
MASTER_SITE = True

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = os.path.join(PROJECT_DIR, 'media')

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/admin/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'gugj0$ix$z=!=5@_ey6e5k5!g(m+zq46agcpgn2a7=j+-e0u4t'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
    'django.template.loaders.eggs.load_template_source',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.auth',
    'django.core.context_processors.i18n',
    'django.core.context_processors.debug',
    'django.core.context_processors.request',
    'django.core.context_processors.media',
)

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'easymode.middleware.LocaleFromUrlMiddleWare',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'easymode.middleware.LocaliseUrlsMiddleware',
)

ROOT_URLCONF = 'example.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(PROJECT_DIR, 'templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'easymode',
    # 'easymode.tests',
    'easymode.easypublisher',
    'django.contrib.admin',
    'reversion',
    'foobar',
    'rosetta',
)

# add containing folder to the path
sys.path.insert(0, os.path.dirname(PROJECT_DIR))



from south.db import db
from django.db import models
from easymode.easypublisher.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'EasyPublisherMetaData'
        db.create_table('easypublisher_easypublishermetadata', (
            ('id', orm['easypublisher.EasyPublisherMetaData:id']),
            ('revision', orm['easypublisher.EasyPublisherMetaData:revision']),
            ('status', orm['easypublisher.EasyPublisherMetaData:status']),
            ('language', orm['easypublisher.EasyPublisherMetaData:language']),
        ))
        db.send_create_signal('easypublisher', ['EasyPublisherMetaData'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'EasyPublisherMetaData'
        db.delete_table('easypublisher_easypublishermetadata')
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'easypublisher.easypublishermetadata': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'default': "'nl'", 'max_length': '5'}),
            'revision': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['reversion.Revision']"}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'draft'", 'max_length': '10'})
        },
        'reversion.revision': {
            'comment': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'date_created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        }
    }
    
    complete_apps = ['easypublisher']



from django.conf.urls.defaults import *
import foobar.views

urlpatterns = patterns('',
    url(r'^$', foobar.views.index, name='index'),
    url(r'^raw\.xml$', foobar.views.raw, name='raw'),
    url(r'^chain\.xml$', foobar.views.chain, name='chain'),
    url(r'^xslt\.xml$', foobar.views.xslt, name='xslt'),
    url(r'^frontend/$', foobar.views.frontend, {'format':'html',}, name='frontend_html'),
    url(r'^frontend/data\.(?P<format>(xml))', foobar.views.frontend, name='frontend'),
)



"""
Contains utils for:

Matching first item in a list that matches some predicate.
Mutex
XML parser/generator that handles unknown entities.
Controlling recursion depth
"""
import errno
import os
import threading
import time
import urlparse
import urllib
from contextlib import contextmanager
from hashlib import sha1

from django.conf import settings


__all__ = (
    # members
    'recursion_depth', 'first_match', 'mutex',
    'SemaphoreException','bases_walker', 'url_add_params',

    # packages
    'languagecode', 'polibext', 'standin', 'template', 'xmlutils',
)

# the recursion level is counted globally and is just an approximation.
# usually if the limit is reached there is something wrong in your app.
# if not, try to set settings.RECURSION_LIMIT
RECURSION_LEVEL_DICT = threading.local()
@contextmanager
def recursion_depth(key):
    """
    A context manager used to guard recursion depth for some function.
    Multiple functions can be kept separately because it will be
    counted per key.
    
    Any exceptions raise in the recursive function will reset the counter,
    because the stack will be unwinded.

    usage::

        with recursion_depth('some_function_name') as recursion_level:
            if recursion_level > getattr(settings, 'RECURSION_LIMIT', sys.getrecursionlimit() / 10):
                raise Exception("Too deep")

            # do some recursive dangerous things.

    :param key: The key under which the recursion depth is kept.
    """
    try:
        if not getattr(RECURSION_LEVEL_DICT, 'key', False):
            RECURSION_LEVEL_DICT.key = 0
        RECURSION_LEVEL_DICT.key += 1
        yield RECURSION_LEVEL_DICT.key
        RECURSION_LEVEL_DICT.key -= 1
    except Exception as e:
        RECURSION_LEVEL_DICT.key = 0
        raise e
    
def first_match(predicate, list):
    """
    returns the first value of predicate applied to list, which
    does not return None
    
    >>> 
    >>> def return_if_even(x): 
    ...     if x % 2 is 0:
    ...         return x
    ...     return None
    >>> 
    >>> first_match(return_if_even, [1, 3, 4, 7])
    4
    >>> first_match(return_if_even, [1, 3, 5, 7])
    >>>
    
    :param predicate: a function that returns None or a value.
    :param list: A list of items that can serve as input to ``predicate``.
    :rtype: whatever ``predicate`` returns instead of None. (or None).
    """
    for item in list:
        val = predicate(item)
        if val is not None:
            return val
    
    return None

class SemaphoreException(Exception):
    """An exception that get thrown when an error occurs in the mutex semphore context"""

class mutex(object):
    """
    A semaphore Context Manager that uses a temporary file for locking.
    Only one thread or process can get a lock on the file at once.
    
    it can be used to mark a block of code as being executed exclusively
    by some thread. see `mutex <http://en.wikipedia.org/wiki/Mutual_exclusion>`_.
    
    usage::
        
        from __future__ import with_statement
        from easymode.utils import mutex
        
        with mutex:
            print "hi only one thread will be executing this block of code at a time."
    
    Mutex raises an :class:`easymode.utils.SemaphoreException` when it has to wait to
    long to obtain a lock or when it can not determine how long it was waiting.
    
    :param max_wait: The maximum amount of seconds the process should wait to obtain\
        the semaphore.
    :param lockfile: The path and name of the pid file used to create the semaphore.
    """
    
    def __init__(self, max_wait=None, lockfile=None):
        # the maximum reasonable time for aprocesstobe
        if max_wait is not None:
            self.max_wait = max_wait
        else:
            self.max_wait = 10
        
        # the location of the lock file
        self.lockfile = lockfile or os.path.join( "/tmp/", sha1(settings.SECRET_KEY).hexdigest() + '.semaphore')
    
    def __enter__(self):
        while True:
            try:
                # if the file exists you can not create it and get an exclusive lock on it
                # this is an atomic operation.
                file_descriptor = os.open(self.lockfile, os.O_EXCL | os.O_RDWR | os.O_CREAT)
                # we created the lockfile, so we're the owner
                break
            except OSError as e:
                if e.errno != errno.EEXIST:
                    # should not occur
                    raise e

            # if we got here the file exists so lets see
            # how long we are waiting for it
            try:
                # the lock file exists, try to stat it to get its age
                # and read it's contents to report the owner PID
                file_contents = open(self.lockfile, "r")
                file_last_modified = os.path.getmtime(self.lockfile)
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise SemaphoreException("%s exists but stat() failed: %s" %                        
                        (self.lockfile, e.strerror)
                    )
                # we didn't create the lockfile, so it did exist, but it's
                # gone now. Just try again
                continue

            # we didn't create the lockfile and it's still there, check
            # its age
            if time.time() - file_last_modified > self.max_wait:
                pid = file_contents.read()
                raise SemaphoreException("%s has been locked for more than " \
                    "%d seconds by PID %s" % (self.lockfile, self.max_wait, pid))

            # it's not been locked too long, wait a while and retry
            file_contents.close()
            time.sleep(1)

        # if we get here. we have the lockfile. Convert the os.open file
        # descriptor into a Python file object and record our PID in it

        file_handle = os.fdopen(file_descriptor, "w")
        file_handle.write("%d" % os.getpid())
        file_handle.close()
    
    def __exit__(self, exc_type, exc_value, traceback):
        #Remove the lockfile, releasing the semaphore for other processes to obtain
        os.remove(self.lockfile)


def bases_walker(cls):
    """
    Loop through all bases of cls
            
    >>> str = u'hai'
    >>> for base in bases_walker(unicode):
    ...     isinstance(str, base)
    True
    True
    
    :param cls: The class in which we want to loop through the base classes.
    """
    for base in cls.__bases__:
        yield base
        for more in bases_walker(base):
            yield more

def url_add_params(url, **kwargs):
    """
    Add parameters to an url
    
    >>> url_add_params('http://example.com/', a=1, b=3)
    'http://example.com/?a=1&b=3'
    >>> url_add_params('http://example.com/?c=8', a=1, b=3)
    'http://example.com/?c=8&a=1&b=3'
    >>> url_add_params('http://example.com/#/irock', a=1, b=3)
    'http://example.com/?a=1&b=3#/irock'
    >>> url_add_params('http://example.com/?id=10#/irock', a=1, b=3)
    'http://example.com/?id=10&a=1&b=3#/irock'
    """
    parsed_url = urlparse.urlsplit(url)
    params = urlparse.parse_qsl(parsed_url.query)
    parsed_url = list(parsed_url)
    
    for pair in kwargs.iteritems():
        params.append(pair)
        
    parsed_url[3] = urllib.urlencode(params)
    return urlparse.urlunsplit(parsed_url)



from django.conf import settings
from django.core.management.base import BaseCommand
from django.db.models import get_models, get_app, get_model
from django.utils.encoding import force_unicode
from django.utils import translation

from easymode import i18n

class Command(BaseCommand):
    help = """
        easy_reset_language <target locale> <app>
        
        will clear all fields for the locale in question so the values will be read from
        the locale again.
        
        example:
        ./manage.py easy_reset_language en myapp.mymodel
        
        This will clear myapp.mymodel in the en locale so all values
        will be fetched by gettext instead of being overridden.
    """
    
    def handle(self, locale, app=None, **options):
        """ command execution """
        translation.activate(settings.LANGUAGE_CODE)
        if app:
            unpack = app.split('.')

            if len(unpack) == 2:
                models = [get_model(unpack[0], unpack[1])]
            elif len(unpack) == 1:
                models = get_models(get_app(unpack[0]))
        else:
            models = get_models()
            
        for model in models:
            if hasattr(model, 'localized_fields'):
                i18n.unregister(model)
                model_full_name = '%s.%s' % (model._meta.app_label, model._meta.module_name)
                update_instances = set()
                messages = []
                
                for instance in model.objects.all():
                    for field in model.localized_fields:
                        local_field_name = "%s_%s" % (field, locale)
                        
                        if  hasattr(instance, local_field_name):
                                local_field_value = getattr(instance, local_field_name)
                                if local_field_value not in (None, u''):
                                    setattr(instance, local_field_name, None)
                                
                                    update_instances.add(instance)
                                    messages.append(u"%s %s %s : %s will become reset to None" % (model_full_name, instance, local_field_name, force_unicode(local_field_value)))

                if len(update_instances):
                    if self.ask_for_confirmation(messages, u'%s.%s' % (model._meta.app_label, model._meta.module_name)):
                        for update_instance in update_instances:
                            print u"resetting %s" % update_instance
                            update_instance.save()
                
                
    def ask_for_confirmation(self, messages, model_full_name):
        print u'\nSummary for "%s":' % model_full_name
        for message in messages:
            print '   %s' % message
        while True:
            prompt = u'\nAre you sure that you want to reset %s: (y/n) [n]: ' % model_full_name
            answer = raw_input(prompt).strip()
            if answer == '':
                return False
            elif answer not in ('y', 'n', 'yes', 'no'):
                print u'Please answer yes or no'
            elif answer == 'y' or answer == 'yes':
                return True
            else:
                return False
            



import warnings
from django.utils.decorators import decorator_from_middleware
from django.test import TestCase
from django.http import HttpResponse
from django.conf import settings

from easymode import middleware
from easymode.utils import languagecode
from easymode.tests.testcases import initdb
from easymode.tests.testutils import TestSettingsManager

__all__ = ('TestLocaliseUrlsMiddleware', 'TestLocaleFromUrlMiddleWare')

@decorator_from_middleware(middleware.LocaliseUrlsMiddleware)
def localise_urls_middleware_view(request):
    return HttpResponse("""
    <a href="/example/modifyme.html">hhh</a>
    <a href="/en/example/staysthesame.html">hhh</a>
    <a href="/xx/notareallangugaecode.html"/>
    <a href="/de/reallanguagecode.html"/>
    """)

@initdb
class TestLocaliseUrlsMiddleware(TestCase):
    """Test the middlewares that come with easymode"""

    def tearDown(self):
        self.settingsManager.revert()
        reload(languagecode)
        
    def test_localise_urls_middleware(self):
        "The LocaliseUrlsMiddleware should insert the languagecode as a slug in all anchor tags"
                
        result = localise_urls_middleware_view(type('RequestMock', tuple(), {'path':'koek', 'LANGUAGE_CODE':'en'}))
                
        self.assertContains(result,'href="/en/example/modifyme.html')
        self.assertContains(result, 'href="/en/example/staysthesame.html')
        self.assertContains(result, 'href="/en/xx/notareallangugaecode.html')
        self.assertContains(result, 'href="/de/reallanguagecode.html"')
        
        result = localise_urls_middleware_view(type('RequestMock', tuple(), {'path':'koek', 'LANGUAGE_CODE':'xx'}))
        
        self.assertContains(result,'href="/xx/example/modifyme.html')
        self.assertContains(result, 'href="/en/example/staysthesame.html')
        self.assertContains(result, 'href="/xx/xx/notareallangugaecode.html')
        self.assertContains(result, 'href="/de/reallanguagecode.html"')


    def test_localise_urls_middleware_use_short_language_codes(self):
        """
        The LocaliseUrlsMiddleware should insert a short version of the
        languagecode as a slug in all anchor tags that do not yet have a
        langugage
        """
        
        self.settingsManager.set(USE_SHORT_LANGUAGE_CODES=True)
        reload(languagecode)

        result = localise_urls_middleware_view(type('RequestMock', tuple(), {'path':'koek', 'LANGUAGE_CODE':'en-us'}))
            
        self.assertContains(result,'href="/en/example/modifyme.html')
        self.assertContains(result, 'href="/en/example/staysthesame.html')
        self.assertContains(result, 'href="/en/xx/notareallangugaecode.html')
        self.assertContains(result, 'href="/de/reallanguagecode.html"')


@decorator_from_middleware(middleware.LocaleFromUrlMiddleWare)
def locale_from_url_middle_ware_view(request):
    return HttpResponse(request.LANGUAGE_CODE)

@initdb
class TestLocaleFromUrlMiddleWare(TestCase):
    "Test the LocaleFromUrlMiddleWare"
    
    def setUp(self):
        self.RequestMockShort = type('RequestMockShort', tuple(), {'path_info':'/en/hi.html', 'set_cookie':lambda x, y, z: x})
        self.RequestMockLong = type('RequestMockLong', tuple(), {'path_info':'/en-us/hi.html', 'set_cookie':lambda x, y, z: x})
    
    def tearDown(self):
        self.settingsManager.revert()
        reload(languagecode)
        reload(middleware)
        
    def test_locale_from_url_middle_ware_normal(self):
        """When USE_SHORT_LANGUAGE_CODES=False languages in urls should be
        resolved as themselves"""
        
        self.settingsManager.set(USE_SHORT_LANGUAGE_CODES=False)
        reload(languagecode)
        reload(middleware)

        result = locale_from_url_middle_ware_view(self.RequestMockShort())
        self.assertEqual(result.content, 'en')
        
        result =  locale_from_url_middle_ware_view(self.RequestMockLong())
        self.assertEqual(result.content, 'en-us')
        
        
    def test_locale_from_url_middle_ware_short(self):
        """When USE_SHORT_LANGUAGE_CODES=True languages in urls should be
        treated as shorthands for longer language codes."""
        
        self.settingsManager.set(USE_SHORT_LANGUAGE_CODES=True,
            LANGUAGE_CODE='de',
            LANGUAGES=(
                ('de', 'German'),
                ('en-us', 'usa'),
            )
        )
        reload(languagecode)
        reload(middleware)
        
        # if the url is prefixed with a known shorthand
        # the language is returned
        result = locale_from_url_middle_ware_view(self.RequestMockShort())
        self.assertEqual(result.content, 'en-us')

        # if the url is not prefixed with a known shorthand
        # the LANGUAGE_CODE is returned.
        result = locale_from_url_middle_ware_view(self.RequestMockLong())
        self.assertEqual(result.content, settings.LANGUAGE_CODE)




import functools

from django import template
from django.conf import settings
from django.conf.urls.defaults import patterns, url
from django.shortcuts import get_object_or_404, render_to_response
from django.utils.translation import ugettext as _
from django.utils.translation import get_language
from django.forms import model_to_dict
from django.forms.formsets import all_valid
from django.db import transaction
from django.http import HttpResponseRedirect
from django.contrib.contenttypes.models import ContentType
from django.contrib.admin import helpers
from django.core.urlresolvers import reverse
from django.utils.html import mark_safe
from django.utils.encoding import force_unicode
from django.utils.dateformat import format

from reversion.admin import VersionAdmin
from reversion.revisions import Version, Revision
import reversion

from easymode.easypublisher.models import EasyPublisherMetaData, EasyPublisherModel
from easymode.tree.admin.relation import ForeignKeyAwareModelAdmin, InvisibleModelAdmin
from easymode.utils import first_match

__all__ = ('EasyPublisher', 'EasyPublisherInvisibleModelAdmin', 'EasyPublisherFKAModelAdmin')

def _eq(self, obj):
    "Helper to have proper equality for models"
    field_names = map(lambda x: x.name, self._meta.fields)
    
    if not field_names:
        raise Exception("%s has no fields or hides them very well" % self)
    
    for name in field_names:
        try:
            if getattr(self, name) != getattr(obj, name):
                return False
        except AttributeError:
            return False
            
    return True

class EasyPublisher(VersionAdmin):
    """
    An admin class that adds approval functionality to the django admin.
    
    Drafts will be saved as revisions using reversion.
    """
    
    object_history_template = "easymode/easypublisher/object_history.html"
    revision_form_template = "easymode/easypublisher/publish_form.html"
    change_form_template = 'easymode/easypublisher/change_form.html'
    
    def get_urls(self):
        """The urls for the publisher"""
        urls = super(EasyPublisher, self).get_urls()
        admin_site = self.admin_site
        opts = self.model._meta
        info = opts.app_label, opts.module_name,
        
        easy_publisher_urls = patterns("",
            url(r"^([^/]+)/drafts/$", admin_site.admin_view(self.drafts_view), name='%s_%s_draftlist' % info),
            url(r"^([^/]+)/drafts/([^/]+)/", admin_site.admin_view(self.publish_view), name='%s_%s_draft' % info),
            url(r"^(.+)/current/$", admin_site.admin_view(self.change_view), {'extra_context':{'current':True}}, name='%s_%s_current' % info)
        )

        return easy_publisher_urls + urls
            
    def drafts_view(self, request, object_id, extra_context=None):
        """Renders the drafts view, listing all drafts"""
        opts = self.model._meta
        action_list = [{"revision": version.revision,
                        "url": reverse("admin:%s_%s_draft" % (opts.app_label, opts.module_name), args=(version.object_id, version.revision.id))}
                       for version in self.get_draft_versions(object_id).select_related("revision")]
        context = {
            "action_list": action_list, 
            "title": _("Unpublished items"), 
            'draft_view':True, 
            'has_draft':self.has_draft(object_id)
        }
        context.update(extra_context or {})
        return super(EasyPublisher, self).history_view(request, object_id, context)
    
    def history_view(self, request, object_id, extra_context=None):
        """Renders the history view, shows the drafts button, but hides the history button"""
        defaults = {
            'has_draft': self.has_draft(object_id)
        }
        defaults.update(extra_context or {})
        return super(EasyPublisher, self).history_view(request, object_id, defaults)
    
    def change_view(self, request, object_id, extra_context=None):
        """Renders the normal editing view, which does not display a draft"""

        latest_draft = self.get_latest_draft(object_id)
        has_publish_perm = request.user.has_perm("easypublisher.can_approve_for_publication")
        context = extra_context or {}

        if latest_draft:
            context['has_draft'] = latest_draft.pk
            
            if not context.get('current', False):
            
                if not has_publish_perm:                
                    return HttpResponseRedirect('drafts/%s/' % latest_draft.pk)
        
        return super(EasyPublisher, self).change_view(request, object_id, context)
        
    @transaction.commit_on_success
    @reversion.revision.create_on_success
    def publish_view(self, request, object_id, revision_id, extra_context=None):
        """
        Displays a draft.
        If you have publishing right, the save button will publish the draft.
        If you don't it will just create another draft.
        """
        
        obj = get_object_or_404(self.model, pk=object_id)
        version = get_object_or_404(Version,
            revision=revision_id,
            object_id=force_unicode(obj.pk),
            content_type=ContentType.objects.get_for_model(obj))
        
        if not version.revision.easypublishermetadata_set.filter(language=request.LANGUAGE_CODE):
            request.user.message_set.create(message=_("There is no draft available for language %s") % request.LANGUAGE_CODE)
            return HttpResponseRedirect('../../current')
        
        # Generate the context.
        context = {
            "title": _("Publish %(name)s") % {"name": self.model._meta.verbose_name},
            "publish":True,
            'has_draft':True,
            'link_current':True,
            'extra':0,
            'revision_id': revision_id,
        }
        context.update(extra_context or {})
        return self.render_revision_form(request, obj, version, context, revert=True)
            
    def save_model(self, request, obj, form, change):
        """
        Saves the model if you have publishing right, but only makes
        a draft in reversion if you don't
        """
        if request.user.has_perm("easypublisher.can_approve_for_publication"):
            obj.save()
        else:
            reversion.revision.add_meta(EasyPublisherMetaData, status='draft', language=request.LANGUAGE_CODE)
            reversion.revision.comment = "Draft"

            if not change and hasattr(obj, 'published'):
                obj.published = False
                obj.save()
                obj.published = True
            
            reversion.revision.post_save_receiver(obj, 0)
                

    def save_formset(self, request, form, formset, change):
        """
        Saves the formset if you have publishing right, but only makes
        a draft in reversion if you don't
        """

        if request.user.has_perm("easypublisher.can_approve_for_publication"):
            formset.save()
        else:
            reversion.revision.add_meta(EasyPublisherMetaData, status='draft', language=request.LANGUAGE_CODE)
            instances = formset.save(commit=False)
            
            for instance in instances:
                if instance.pk is None:
                    # reversion puts these models in a set(). If we don't define
                    # proper equality for them, they will all count as equal, since
                    # their id's are all the same. This should not have to be done
                    # for every object, but only once for every type of object.
                    # however, it really doesn't matter that much for performance
                    # so this is how it is.
                    setattr(instance.__class__, '__eq__', _eq)
                reversion.revision.post_save_receiver(instance, 0)
            
    def has_draft(self, object_id):
        """Find out if there is a draft version of this model"""
        return len(self.get_draft_revisions(object_id))
    
    def get_draft_revisions(self, object_id):
        """
        retrieve all revision marked as draft that belong to object_id.
        """
        content_type = ContentType.objects.get_for_model(self.model)
        return Revision.objects.filter(
            version__object_id=object_id, 
            version__content_type=content_type,
            easypublishermetadata__status='draft',
            easypublishermetadata__language=get_language()
        ).select_related().distinct()
    
    def get_draft_versions(self, object_id):
        """
        Retrieve all drafts that belong to this item
        """
        content_type = ContentType.objects.get_for_model(self.model)
        versions = Version.objects.filter(
            revision__easypublishermetadata__status='draft',
            revision__easypublishermetadata__language=get_language(),
            object_id=object_id,
            content_type=content_type
        ).distinct()
        
        return versions
        
    def get_latest_draft_revision(self, object_id):
        """
        Retrieve latest draft revision that belongs to object_id.
        """
        revisions = self.get_draft_revisions(object_id)
        num_revisions = len(revisions)
        if num_revisions:
            return revisions[num_revisions-1]
        
        return None
    
    def get_latest_draft(self, object_id):
        """
        Retrieves the latest draft that belongs to object_id.
        """
        latest_revision = self.get_latest_draft_revision(object_id)
        return latest_revision
    
    def update_draft(self, version, request):
        """Update the status of the draft belonging to this version"""
        for metadata in version.revision.easypublishermetadata_set.all():
            if request.user.has_perm("easypublisher.can_approve_for_publication"):                
                metadata.status = 'published'
                # save all other drafts for this object as declined, because we
                # chose to save a different one
                for other in EasyPublisherMetaData.objects.filter(
                    revision__version__object_id=version.object_id, 
                    revision__version__content_type=version.content_type):
                    other.status = 'declined'
                    other.save()
            else:
                metadata.status = 'updated'
            metadata.save()
        
    
    ##     ##    ###    ##     ## 
    ##     ##   ## ##    ##   ##  
    ##     ##  ##   ##    ## ##   
    ######### ##     ##    ###    
    ##     ## #########   ## ##   
    ##     ## ##     ##  ##   ##  
    ##     ## ##     ## ##     ## 
    # this is copy pasted because reversion does not let us do anything with the 
    # messages that go after the you know what.
    # please remove this if nolonger needed
    def render_revision_form(self, request, obj, version, context, revert=False, recover=False):
        """Renders the object revision form."""
        model = self.model
        opts = model._meta
        object_id = obj.pk
        # Generate the model form.
        ModelForm = self.get_form(request, obj)
        formsets = []
        if request.method == "POST":
            # This section is copied directly from the model admin change view
            # method.  Maybe one day there will be a hook for doing this better.
            form = ModelForm(request.POST, request.FILES, instance=obj, initial=self.get_revision_form_data(request, obj, version))
            if form.is_valid():
                form_validated = True
                new_object = self.save_form(request, form, change=True)
            else:
                form_validated = False
                new_object = obj
            prefixes = {}
            for FormSet, inline in zip(self.get_formsets(request, new_object),
                                       self.inline_instances):
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                
                formset = FormSet(request.POST, request.FILES,
                                  instance=new_object, prefix=prefix,
                                  queryset=inline.queryset(request))

                # _ _ _ ____ ___ ____ _  _    ____ _  _ ___ 
                # | | | |__|  |  |    |__|    |  | |  |  |  
                # |_|_| |  |  |  |___ |  |    |__| |__|  |
                # this is not copy pasted:
                # Strip extra empty forms from the formset.
                empty_forms = []
                post_keys = request.POST.keys()
                for f in formset.forms:
                    # the forms that can be removed, are not in the request.POST
                    # we can find them by their prefix
                    matches_prefix = lambda var: var.find(f.prefix) == 0 or None
                    if first_match(matches_prefix, post_keys) is None:
                        empty_forms.append(f)
                    else:
                        # the form must be cleaned, which they aren't yet because
                        # we just created them
                        f.full_clean()
                
                # modify form settings of formset.
                num_forms = formset.total_form_count() - len(empty_forms)
                formset.forms = filter(lambda x: x not in empty_forms, formset.forms)
                formset.total_form_count = lambda: num_forms
                # end of non copy pasted piece
                
                # Add this hacked formset to the form.                
                formsets.append(formset)
            if all_valid(formsets) and form_validated:
                self.save_model(request, new_object, form, change=True)
                form.save_m2m()
                for formset in formsets:
                    self.save_formset(request, form, formset, change=True)
                
                # _ _ _ ____ ___ ____ _  _    ____ _  _ ___ 
                # | | | |__|  |  |    |__|    |  | |  |  |  
                # |_|_| |  |  |  |___ |  |    |__| |__|  |
                # this is not copy pasted:
                self.update_draft(version, request)
                
                change_message = _(u"Publisher message %(datetime)s") % {"datetime": format(version.revision.date_created, _(settings.DATETIME_FORMAT))}
                self.log_change(request, new_object, change_message)
                self.message_user(request, _(u'%(model)s "%(name)s" publisher message.') % {"model": force_unicode(opts.verbose_name), "name": unicode(obj)})

                # end of not copy pasted code

                # Redirect to the model change form.
                if revert:
                    return HttpResponseRedirect("../../")
                elif recover:
                    return HttpResponseRedirect("../../%s/" % object_id)
                else:
                    assert False
        else:
            # This is a mutated version of the code in the standard model admin
            # change_view.  Once again, a hook for this kind of functionality
            # would be nice.  Unfortunately, it results in doubling the number
            # of queries required to construct the formets.
            form = ModelForm(instance=obj, initial=self.get_revision_form_data(request, obj, version))
            prefixes = {}
            revision_versions = version.revision.version_set.all()
            for FormSet, inline in zip(self.get_formsets(request, obj), self.inline_instances):
                # This code is standard for creating the formset.
                prefix = FormSet.get_default_prefix()
                prefixes[prefix] = prefixes.get(prefix, 0) + 1
                if prefixes[prefix] != 1:
                    prefix = "%s-%s" % (prefix, prefixes[prefix])
                formset = FormSet(instance=obj, prefix=prefix,
                                  queryset=inline.queryset(request))
                # Now we hack it to push in the data from the revision!
                try:
                    fk_name = FormSet.fk.name
                except AttributeError:
                    # This is a GenericInlineFormset, or similar.
                    fk_name = FormSet.ct_fk_field.name
                
                # _ _ _ ____ ___ ____ _  _    ____ _  _ ___ 
                # | | | |__|  |  |    |__|    |  | |  |  |  
                # |_|_| |  |  |  |___ |  |    |__| |__|  |
                # this is not copy pasted:
                related_versions = [(related_version.object_id, related_version)
                                         for related_version in revision_versions
                                         if ContentType.objects.get_for_id(related_version.content_type_id).model_class() == FormSet.model
                                         and unicode(related_version.field_dict[fk_name]) == unicode(object_id)]
                
                # all items that do not have their id filled in must stay.
                # if they do in fact have an id, we only want the last one,
                # no duplicates.
                related_versions = [(key, value) for (key, value) in related_versions if key == 'None'] + \
                    dict([(key, value) for (key, value) in related_versions if key != 'None']).items()
                
                initial = []
                for related_obj in formset.queryset:
                    related_versions_dict = dict(related_versions)
                    if unicode(related_obj.pk) in related_versions_dict:
                        initial.append(related_versions_dict.pop(unicode(related_obj.pk)).field_dict)
                    else:
                        initial_data = model_to_dict(related_obj)
                        initial_data["DELETE"] = True
                        initial.append(initial_data)
                for (key, related_version) in related_versions:
                    initial_row = related_version.field_dict
                    try:
                        del initial_row["id"]
                    except KeyError:
                        # when multiple inlines with the same model but a different
                        # querysets are on a page, it could happen that we allready
                        # deleted the id.
                        pass
                    
                    # we didn't convert related_versions to a dict so there can be
                    # duplicate keys in the thingy
                    if not initial_row in initial:
                        initial.append(initial_row)
                
                # end of non copy pasted piece
                
                # Reconstruct the forms with the new revision data.
                formset.initial = initial
                formset.forms = [formset._construct_form(n) for n in xrange(len(initial))]
                # Add this hacked formset to the form.
                formsets.append(formset)
        # Generate admin form helper.
        adminForm = helpers.AdminForm(form, self.get_fieldsets(request, obj),
            self.prepopulated_fields, self.get_readonly_fields(request, obj),
            model_admin=self)
        media = self.media + adminForm.media
        # Generate formset helpers.
        inline_admin_formsets = []
        for inline, formset in zip(self.inline_instances, formsets):
            fieldsets = list(inline.get_fieldsets(request, obj))
            readonly = list(inline.get_readonly_fields(request, obj))
            inline_admin_formset = helpers.InlineAdminFormSet(inline, formset,
                fieldsets, readonly, model_admin=self)
            inline_admin_formsets.append(inline_admin_formset)
            media = media + inline_admin_formset.media
        # Generate the context.
        context.update({"adminform": adminForm,
                        "object_id": object_id,
                        "original": obj,
                        "is_popup": False,
                        "media": mark_safe(media),
                        "inline_admin_formsets": inline_admin_formsets,
                        "errors": helpers.AdminErrorList(form, formsets),
                        "app_label": opts.app_label,
                        "add": False,
                        "change": True,
                        "has_add_permission": self.has_add_permission(request),
                        "has_change_permission": self.has_change_permission(request, obj),
                        "has_delete_permission": self.has_delete_permission(request, obj),
                        "has_file_field": True,
                        "has_absolute_url": getattr(self.model, 'get_absolute_url', False),
                        "ordered_objects": opts.get_ordered_objects(),
                        "form_url": mark_safe(request.path),
                        "opts": opts,
                        "content_type_id": ContentType.objects.get_for_model(self.model).id,
                        "save_as": False,
                        "save_on_top": self.save_on_top,
                        "changelist_url": reverse("admin:%s_%s_changelist" % (opts.app_label, opts.module_name)),
                        "change_url": reverse("admin:%s_%s_change" % (opts.app_label, opts.module_name), args=(obj.pk,)),
                        "history_url": reverse("admin:%s_%s_history" % (opts.app_label, opts.module_name), args=(obj.pk,)),
                        "recoverlist_url": reverse("admin:%s_%s_recoverlist" % (opts.app_label, opts.module_name))})
        
        # Render the form.
        if revert:
            form_template = self.revision_form_template
        elif recover:
            form_template = self.recover_form_template
        else:
            assert False
        return render_to_response(form_template, context, template.RequestContext(request))

def _add_foreign_key_aware_model_admin_behaviour(method):
    """
    Is used to add some stuff of ForeignKeyAwareModelAdmin to
    EasyPublisher views.
    """
    @functools.wraps(method)
    def altered_view(self, request, object_id, extra_context=None):
        inline_links = {}
        inline_links['extra_forms'] = self.extra_forms(object_id)

        # retrieve link to parent for breadcrumb path
        inline_links.update(self._get_parent_link(object_id))
        if extra_context:
            inline_links.update(extra_context)
        
        return method(self, request, object_id, inline_links)
    
    return altered_view

class EasyPublisherFKAModelAdmin(EasyPublisher, ForeignKeyAwareModelAdmin):
    """fixes the collision between EasyPublisher, ForeignKeyAwareModelAdmin"""

    change_form_template = 'easymode/easypublisher/change_form_with_related_links.html'
    revision_form_template = 'easymode/easypublisher/publish_form_with_related_links.html'
    def get_model_perms(self, request):
        perms = super(EasyPublisherFKAModelAdmin, self).get_model_perms(request)
        perms['invisible_in_admin'] = self.invisible_in_admin
        return perms
    
    def publish_view(self, request, object_id, version_id, extra_context=None):
        inline_links = {}
        inline_links['extra_forms'] = self.extra_forms(object_id)

        # retrieve link to parent for breadcrumb path
        inline_links.update(self._get_parent_link(object_id))
        if extra_context:
            inline_links.update(extra_context)
            
        return EasyPublisher.publish_view(self, request, object_id, version_id, inline_links)
        
    change_view = _add_foreign_key_aware_model_admin_behaviour(EasyPublisher.change_view)
    drafts_view = _add_foreign_key_aware_model_admin_behaviour(EasyPublisher.drafts_view)
    history_view = _add_foreign_key_aware_model_admin_behaviour(EasyPublisher.history_view)

def _add_invisible_model_admin_behaviour(method):
    """
    Is used to add some stuff of InvisibleModelAdmin to
    EasyPublisher views.
    """
    @functools.wraps(method)
    def altered_view(self, request, object_id, extra_context=None):
        # retrieve link to parent for breadcrumb path
        defaults = self._get_parent_link(object_id)

        if extra_context:
            defaults.update(extra_context)
    
        return method(self, request, object_id, defaults)
    
    return altered_view

class EasyPublisherInvisibleModelAdmin(EasyPublisher, InvisibleModelAdmin):
    """
    fixes the collision between EasyPublisher and InvisibleModelAdmin
    
    see :mod:`easymode.tree.admin.relation`
    """
        
    def get_model_perms(self, request):
        perms = super(EasyPublisherInvisibleModelAdmin, self).get_model_perms(request)
        perms['invisible_in_admin'] = self.invisible_in_admin
        return perms
    
    def publish_view(self, request, object_id, version_id, extra_context=None):
        """docstring for publish_view"""
        defaults = self._get_parent_link(object_id)

        if extra_context:
            defaults.update(extra_context)
            
        return EasyPublisher.publish_view(self, request, object_id, version_id, defaults)      
         
    change_view = _add_invisible_model_admin_behaviour(EasyPublisher.change_view)
    drafts_view = _add_invisible_model_admin_behaviour(EasyPublisher.drafts_view)
    history_view = _add_invisible_model_admin_behaviour(EasyPublisher.history_view)



from django.conf.urls.defaults import *
from django.conf import settings
# Uncomment the next two lines to enable the admin:
from django.contrib import admin
from easymode.utils.languagecode import get_language_codes_as_disjunction


admin.autodiscover()

languages = get_language_codes_as_disjunction()

urlpatterns = patterns('',
    # Example:
    # (r'^example/', include('example.foo.urls')),

    # Uncomment the admin/doc line below and add 'django.contrib.admindocs' 
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    (r'^admin/', include(admin.site.urls)),
    (r'^', include('foobar.urls')),
    
    # These are the url patterns for the internationalised urls.
    ('^(%(languages)s)/admin/' % locals(), include(admin.site.urls)),
    ('^(%(languages)s)/' % locals(), include('foobar.urls'))
    
)

if settings.DEBUG:
    urlpatterns += patterns('',
        (r'^media/(?P<path>.*)$', 'django.views.static.serve', {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),        
    )


"""
Easymode is an aspect oriented toolkit that helps making xml based flash
websites easy. The tools included in the toolkit to help you make these kind of
sites include:

* Internationalization of models, with admin support
* Translation of model data using gettext
* Automatic generation of xml from model trees using xslt
* Admin support for model trees with more than 2 levels of related items
* Basic approval support for models

documentation at http://packages.python.org/django-easymode/

release notes at http://packages.python.org/django-easymode/changes.html
"""

__all__ = ('admin', 'debug', 'easypublisher', 'i18n', 'management',
    'templatetags', 'tree', 'utils', 'xslt')



from django.contrib.admin.views.decorators import staff_member_required

from easymode.utils import url_add_params
from easymode.views import preview


@staff_member_required
def preview_draft(request, content_type_id, object_id, revision_id):
    """
    A shortcut view that redirects to the absolute url of the object that the
    *content_type_id* and *object_id* define uniquely. An extra parameter is added
    to this absolute url;
    
    ?preview=*revision_id*
    
    Which can be used to show the data which is in revision *revision_id* in your
    view instead of the regular data.
    """
    response = preview(request, content_type_id, object_id)
    response['Location'] = url_add_params(response['Location'], preview=revision_id)
    return response



"""
Replacements for django middlewares that allow you to see what is put in to the cache
and comes out of the cache.
"""
import logging

from django.conf import settings
from django.core.cache import cache
from django.utils.cache import get_cache_key, learn_cache_key, patch_response_headers, get_max_age, _generate_cache_header_key

def get_cache_key_parameters(request):
    result = []
    key_prefix = settings.CACHE_MIDDLEWARE_KEY_PREFIX
    result.append("key_prefix: %s" % key_prefix)
    cache_key = _generate_cache_header_key(key_prefix, request)
    headerlist = cache.get(cache_key, None)
    if headerlist:
        for header in headerlist:
            value = request.META.get(header, None)
            if value is not None:
                result.append("%s = %s" % (header, value))
    return result
    
class DebugUpdateCacheMiddleware(object):
    """
    Same as :class:`~django.middleware.cache.django.middleware.cache.UpdateCacheMiddleware` but is shows you
    when something is put into the cache.
    """
    def __init__(self):
        self.cache_timeout = settings.CACHE_MIDDLEWARE_SECONDS
        self.key_prefix = settings.CACHE_MIDDLEWARE_KEY_PREFIX
        self.cache_anonymous_only = getattr(settings, 'CACHE_MIDDLEWARE_ANONYMOUS_ONLY', False)

    def process_response(self, request, response):
        """Sets the cache, if needed."""
        if not hasattr(request, '_cache_update_cache') or not request._cache_update_cache:
            # We don't need to update the cache, just return.
            return response
        if request.method != 'GET':
            # This is a stronger requirement than above. It is needed
            # because of interactions between this middleware and the
            # HTTPMiddleware, which throws the body of a HEAD-request
            # away before this middleware gets a chance to cache it.
            return response
        if not response.status_code == 200:
            return response
        # Try to get the timeout from the "max-age" section of the "Cache-
        # Control" header before reverting to using the default cache_timeout
        # length.
        timeout = get_max_age(response)
        if timeout == None:
            timeout = self.cache_timeout
        elif timeout == 0:
            # max-age was set to 0, don't bother caching.
            return response
        patch_response_headers(response, timeout)
        if timeout:
            cache_key = learn_cache_key(request, response, timeout, self.key_prefix)
            cache.set(cache_key, response, timeout)
            logging.debug("UpdateCacheMiddleware: setting %s -> %s params are: %s" % (cache_key, request.path, get_cache_key_parameters(request)))
        return response

class DebugFetchFromCacheMiddleware(object):
    """
    Same as :class:`~django.middleware.cache.django.middleware.cache.FetchFromCacheMiddleware` but it shows
    you what it is retrieving from the cache if it can find something.
    """
    def __init__(self):
        self.cache_timeout = settings.CACHE_MIDDLEWARE_SECONDS
        self.key_prefix = settings.CACHE_MIDDLEWARE_KEY_PREFIX
        self.cache_anonymous_only = getattr(settings, 'CACHE_MIDDLEWARE_ANONYMOUS_ONLY', False)

    def process_request(self, request):
        """
        Checks whether the page is already cached and returns the cached
        version if available.
        """
        if self.cache_anonymous_only:
            assert hasattr(request, 'user'), "The Django cache middleware with CACHE_MIDDLEWARE_ANONYMOUS_ONLY=True requires authentication middleware to be installed. Edit your MIDDLEWARE_CLASSES setting to insert 'django.contrib.auth.middleware.AuthenticationMiddleware' before the CacheMiddleware."

        if not request.method in ('GET', 'HEAD') or request.GET:
            request._cache_update_cache = False
            return None # Don't bother checking the cache.

        if self.cache_anonymous_only and request.user.is_authenticated():
            request._cache_update_cache = False
            return None # Don't cache requests from authenticated users.

        cache_key = get_cache_key(request, self.key_prefix)
        if cache_key is None:
            request._cache_update_cache = True
            return None # No cache information available, need to rebuild.

        response = cache.get(cache_key, None)
        if response is None:
            logging.debug("FetchFromCacheMiddleware: %s is %s.  paramters(%s)" % ( cache_key, request.path, get_cache_key_parameters(request)))
            request._cache_update_cache = True
            return None # No cache information available, need to rebuild.
        logging.debug("Found %s -> %s in the cache. parameters(%s)" % (cache_key, request.path, get_cache_key_parameters(request)))
        request._cache_update_cache = False
        return response

class DebugCacheMiddleware(DebugUpdateCacheMiddleware, DebugFetchFromCacheMiddleware):
    """
    combines :class:`~easymode.debug.middleware.DebugFetchFromCacheMiddleware` and 
    :class:`easymode.debug.middleware.DebugUpdateCacheMiddleware`
    """
    def __init__(self, cache_timeout=None, key_prefix=None, cache_anonymous_only=None):
        self.cache_timeout = cache_timeout
        if cache_timeout is None:
            self.cache_timeout = settings.CACHE_MIDDLEWARE_SECONDS
        self.key_prefix = key_prefix
        if key_prefix is None:
            self.key_prefix = settings.CACHE_MIDDLEWARE_KEY_PREFIX
        if cache_anonymous_only is None:
            self.cache_anonymous_only = getattr(settings, 'CACHE_MIDDLEWARE_ANONYMOUS_ONLY', False)
        else:
            self.cache_anonymous_only = cache_anonymous_only
